-- CSunitBorrow
-- Author: knightmare
-- DateCreated: 9/29/2012 12:56:01 PM
--------------------------------------------------------------

GameEvents.SetPopulation.Add(function(iX, iY, oldPopulation, newPopulation) 

    local plot = Map.GetPlot(iX, iY);
    local city = plot:GetPlotCity();

    if (city:IsRazing()) then

		if (newPopulation > 0) then

			if (GenerateTreasureFromRazedCity(Game:GetHandicapType(), city:GetOwner(), city:GetPreviousOwner())) then
				local player = Players[city:GetOwner()];
				local iUnitID;
				iUnitID = GameInfoTypes["UNIT_TREASURE"];
				player:InitUnit (iUnitID, iX, iY);
	    
				local text;
				local heading;
				text = Locale.ConvertTextKey("TXT_KEY_NEWWORLD_SCENARIO_TREASURE_RAZED_CITY", city:GetName());
				heading = Locale.ConvertTextKey("TXT_KEY_NEWWORLD_SCENARIO_TREASURE_FOUND");
     				player:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, text, heading);
			end
     		end
    end

    return true;
end);

function GenerateTreasureFromRazedCity (iHandicap, iNewOwner, iOldOwner)

	print("GenerateTreasureFromRazedCity");

	local bEuroNativeOK = false;
	local bDifficultyOK = false;
	
	local pPlayer1 = Players[iNewOwner];
	local pPlayer2 = Players[iOldOwner];
	
	if (pPlayer1:GetStartingPlot():GetX() < 50 and pPlayer2:GetStartingPlot():GetX() > 50) then
		bEuroNativeOK = true;
	elseif (pPlayer2:GetStartingPlot():GetX() < 50 and pPlayer1:GetStartingPlot():GetX() > 50) then
		bEuroNativeOK = true;
	end
	
	if (not bEuroNativeOK) then
		return false;
	end
	
	if( pPlayer1:IsMinorCiv() ) then
		return false;
	end
	
	if (iHandicap <= 3 or iNewOwner > 0) then
		bDifficultyOK = true;
	elseif (iHandicap == 4) then
        if (Game.Rand(100, "Rolling for treasure") < 80) then
			bDifficultyOK = true;
		end
	elseif (iHandicap == 5) then
        if (Game.Rand(100, "Rolling for treasure") < 70) then
			bDifficultyOK = true;
		end
	elseif (iHandicap == 6) then
        if (Game.Rand(100, "Rolling for treasure") < 60) then
			bDifficultyOK = true;
		end
	else
        if (Game.Rand(100, "Rolling for treasure") < 50) then
			bDifficultyOK = true;
		end
	end		

	if (bDifficultyOK) then
		return true;
	else
		return false;
	end
end

GameEvents.UnitSetXY.Add(function(iPlayer, iUnitID, iX, iY) 

    local plot = Map.GetPlot(iX, iY);
	local player = Players[iPlayer];
	local unit = player:GetUnitByID(iUnitID);
	if (unit == nil or unit:IsDelayedDeath()) then
	    return false;
	end

	-- Treasure unit that has reached capital?
	if (unit:GetUnitType() == GameInfoTypes["UNIT_TREASURE"]) then

		-- At my capital?
		local capital = player:GetCapitalCity();
		if (capital ~= nil and capital:GetX() == iX and capital:GetY() == iY) then

			unit:Kill(true, -1);

		    -- Grant gold and VP
			local iGoldReceived = 100;
			local iVPReceived = 10;
			player:ChangeGold(iGoldReceived);
			player:ChangeScoreFromFutureTech(iVPReceived);

			-- Put out notification
			local text;
			local heading;
			text = Locale.ConvertTextKey("TXT_KEY_NEWWORLD_SCENARIO_TREASURE_RETURNED", capital:GetName(), iGoldReceived, iVPReceived);
			heading = Locale.ConvertTextKey("TXT_KEY_NEWWORLD_SCENARIO_TREASURE_RETURNED_SHORT");
     			player:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, text, heading);

			if(iPlayer == Game.GetActivePlayer()) then
				if(capital:GetX() > 50) then
					UI.UnlockAchievement("ACHIEVEMENT_SCENARIO_02_RETURN_TREASURE");
				end
			end
		end
	end

    return true;
end);


GameEvents.PlayerDoTurn.Add(function(iPlayer) 

	-- Only on first player's turn
	if (iPlayer > 0) then
		return false;
	end

	--(KLUDGE - using an unused modern era CvGame data field as a counter)
	local iGenerateTreasureTurn = Game:GetNoNukesCount();
	local iGameTurn = Game.GetGameTurn();    
	local iFeatureID;

	-- Time to generate a treasure?
	if (iGameTurn > iGenerateTreasureTurn) then
		
		iFeatureID = GameInfoTypes["FEATURE_EL_DORADO"];
	    SpawnTreasure(iFeatureID);

		iFeatureID = GameInfoTypes["FEATURE_POTOSI"];
	    SpawnTreasure(iFeatureID);

		iFeatureID = GameInfoTypes["FEATURE_FUJI"];
	    SpawnTreasure(iFeatureID);

		iFeatureID = GameInfoTypes["FEATURE_FOUNTAIN_YOUTH"];
	    SpawnTreasure(iFeatureID);
			
		-- Reset turn of next treasure generation
		local iRandom = Game.Rand(20, "Delay between natural wonder treasures");						
		Game:ChangeNoNukesCount(8 + iRandom);
	end
	return false;
end);

function SpawnTreasure (iFeatureID)

	print("SpawnTreasure");

	for iPlotLoop = 0, Map.GetNumPlots()-1, 1 do
		pPlot = Map.GetPlotByIndex(iPlotLoop);
		if (pPlot:GetFeatureType() == iFeatureID) then

			-- Found the natural wonder, are there tiles owned by a major civ adjacent?
			local iNWOwner = pPlot:GetOwner();
			local numDirections = DirectionTypes.NUM_DIRECTION_TYPES;
			for direction = 0, numDirections - 1, 1 do

				local adjacentPlot = Map.PlotDirection(pPlot:GetX(), pPlot:GetY(), direction);
				if (adjacentPlot ~= nil) then

				    local iPlayer = adjacentPlot:GetOwner();
					if (iPlayer ~= -1) then

						local pPlayer = Players[iPlayer];
						if (pPlayer:IsAlive() and not pPlayer:IsMinorCiv() and (iPlayer == iNWOwner or iNWOwner == -1 or iNWOwner > 7)) then
						
							-- Found one, reward it with a treasure and we're done
							local iUnitID;
							iUnitID = GameInfoTypes["UNIT_TREASURE"];
							pPlayer:InitUnit (iUnitID, adjacentPlot:GetX(), adjacentPlot:GetY());
	    
							-- Generate message if human
							if (iPlayer == 0) then
								local text;
								local heading;
								local name = Locale.ConvertTextKey( GameInfo.Features[iFeatureID].Description );			
								text = Locale.ConvertTextKey("TXT_KEY_NEWWORLD_SCENARIO_TREASURE_NATURAL_WONDER", name);
								heading = Locale.ConvertTextKey("TXT_KEY_NEWWORLD_SCENARIO_TREASURE_FOUND");
     							pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, text, heading);
							end

							return true;
						end
					end
				end
			end
		end
	end
end

function OnCityCaptured (playerID, bCapital, iX, iY, newPlayerID)

	print("OnCityCaptured");
	
    local newPlayer = Players[newPlayerID];
    local oldPlayer = Players[playerID];
	local plot = Map.GetPlot(iX, iY);
    local city = plot:GetPlotCity();
	
	local bEuroNativeOK = false;
	
	-- Block capital capture treasure from Euro taking Euro capital
	if (newPlayer:GetStartingPlot():GetX() < 50) then
		bEuroNativeOK = true;
	elseif (oldPlayer:GetStartingPlot():GetX() < 50 and newPlayer:GetStartingPlot():GetX() > 50) then
		bEuroNativeOK = true;
	end
	
	if( newPlayer:IsMinorCiv() ) then
		return;
	end

    -- Was it a Major?
    if (not oldPlayer:IsMinorCiv()) then

		if (bCapital and city:IsOriginalCapital() and bEuroNativeOK) then

			local iUnitID;
			iUnitID = GameInfoTypes["UNIT_TREASURE"];

			local unit; 
			unit = newPlayer:InitUnit (iUnitID, iX, iY);
			unit:JumpToNearestValidPlot();
			unit = newPlayer:InitUnit (iUnitID, iX, iY);
			unit:JumpToNearestValidPlot();
			unit = newPlayer:InitUnit (iUnitID, iX, iY);
			unit:JumpToNearestValidPlot();

			-- Put up dialog box
			local popupInfo = {
				Data1 = 500,
				Type = ButtonPopupTypes.BUTTONPOPUP_TEXT,
			}
			popupInfo.Text = Locale.ConvertTextKey("TXT_KEY_NEWWORLD_SCENARIO_TREASURE_CAPTURED_CITY", city:GetName());
            UI.AddPopup(popupInfo);
		end	
	else 
		if( city:IsOriginalCapital() ) then
			local iUnitID;
			iUnitID = GameInfoTypes["UNIT_TREASURE"];

			local unit; 
			unit = newPlayer:InitUnit (iUnitID, iX, iY);
			unit:JumpToNearestValidPlot();

			-- Put up dialog box
			local popupInfo = {
				Data1 = 500,
				Type = ButtonPopupTypes.BUTTONPOPUP_TEXT,
			}
			popupInfo.Text = Locale.ConvertTextKey("TXT_KEY_NEWWORLD_SCENARIO_TREASURE_RAZED_CITY", city:GetName());
            UI.AddPopup(popupInfo);
		end
    end
end

GameEvents.CityCaptureComplete.Add(OnCityCaptured);
